#pragma once
#include "public.sdk/source/vst/vstparameters.h"
#include "pluginterfaces/base/ibstream.h"
#include "pluginterfaces/vst/ivstparameterchanges.h"
#include "public.sdk/source/vst/vstaudioeffect.h"
#include "vstgui/plugin-bindings/vst3editor.h"
#include "pluginterfaces/base/ustring.h"

namespace Steinberg {
namespace Vst {
namespace Sock2VST3 {

	class VoltOctaveParameter : public Parameter
	{
	public:
		VoltOctaveParameter(const TChar* title, ParamID tag, const TChar* units = 0,
			ParamValue minPlain = 0., ParamValue maxPlain = 1., ParamValue defaultValuePlain = 0.,
			int32 stepCount = 0, int32 flags = ParameterInfo::kCanAutomate, UnitID unitID = kRootUnitId);

		virtual void toString(ParamValue normValue, String128 string) const;
		virtual bool fromString(const TChar* string, ParamValue& normValue) const;
		virtual ParamValue toPlain(ParamValue _valueNormalized) const;
		virtual ParamValue toNormalized(ParamValue plainValue) const;
		virtual ParamValue getMin() const { return minPlain; }
		virtual void setMin(ParamValue value) { minPlain = value; }
		virtual ParamValue getMax() const { return maxPlain; }
		virtual void setMax(ParamValue value) { maxPlain = value; }

	protected:
		ParamValue minPlain;
		ParamValue maxPlain;

		// cooked to VA Scaled 0->1 param
		inline float calcVoltOctaveParameter(float fCookedParam) const
		{
			double dOctaves = log2(getMax() / getMin());
			return log2(fCookedParam / getMin()) / dOctaves;
		}

		// fPluginValue = 0->1
		// returns VA scaled version 0->1
		inline float calcVoltOctavePluginValue(float fPluginValue) const
		{
			double dOctaves = log2(getMax() / getMin());
			float fDisplay = getMin()*exp2(fPluginValue*dOctaves);
			float fDiff = getMax() - getMin();
			return (fDisplay - getMin()) / fDiff;
		}
	};

}}}